//-----------------------------------------------------------------------------
//
//  Zed_DVI.v : zed zynq-7020 DVI test module
//
//  LICENSE : "as-is"
//
//------------------------------------------------------------------------------

`define reso_640x480
//`define reso_800x600
//`define reso_1024x768

module Zed_DVI(

	input			rst,		// reset
	input			clk100,		// in    100MHz

	output			ch0p,		// out   [PMOD] jc1p (1)
	output			ch0n,		// out   [PMOD] jc1n (2)
	output			ch1p,		// out   [PMOD] jc2p (3)
	output			ch1n,		// out   [PMOD] jc2n (4)
	output			ch2p,		// out   [PMOD] jc3p (7)
	output			ch2n,		// out   [PMOD] jc3n (8)
	output			ch3p,		// out   [PMOD] jc4p (9)
	output			ch3n,		// out   [PMOD] jc4n (10)

	output	[7:0]	ld			// out   led[7:0]

);

	wire	RST;
	wire	RST_N;
	wire	RST2_N;
	wire	RESET_N;

	wire	GCLK100;
	wire	CLK64;
	wire	TX_CLK;
	wire	TX_CLKx5;


	assign RST_N = !rst;
	assign RST   =  rst;



IBUFG IBUFG_GCLK100(.I(clk100),.O(GCLK100));


`ifdef reso_640x480

	// VGA(25MHz)
xil_clk_wiz_v3_5_100x25x125 clkgen_100x25x125(
	.CLK_IN1(GCLK100),
	.CLK_OUT1(TX_CLK),
	.CLK_OUT2(TX_CLKx5),
	.RESET(RST),
	.LOCKED(RESET_N)
);

	assign RST2_N = !rst;

`endif

`ifdef reso_800x600

	// SVGA(40MHz)
xil_clk_wiz_v3_5_100x40x200 clkgen_100x40x200(
	.CLK_IN1(GCLK100),
	.CLK_OUT1(TX_CLK),
	.CLK_OUT2(TX_CLKx5),
	.RESET(RST),
	.LOCKED(RESET_N)
);

	assign RST2_N = !rst;

`endif

`ifdef reso_1024x768

xil_clk_wiz_v3_5_100x64 clkgen_100x64(
	.CLK_IN1(GCLK100),
	.CLK_OUT1(CLK64),
	.RESET(RST),
	.LOCKED(RST2_N)
);

xil_clk_wiz_v3_5_64x64x320 clkgen_64x64x320(
	.CLK_IN1(CLK64),
	.CLK_OUT1(TX_CLK),
	.CLK_OUT2(TX_CLKx5),
	.RESET(!RST2_N),
	.LOCKED(RESET_N)
);

`endif


	wire	TG_HS;
	wire	TG_VS;
	wire	TG_DE;

	wire	[7:0] TX_RED;
	wire	[7:0] TX_GRN;
	wire	[7:0] TX_BLU;
	wire	TX_HS;
	wire	TX_VS;
	wire	TX_DE;

timegen #(

`ifdef reso_640x480

		// VESA CVT : 0.31M3-R 25MHz 640x480@60Hz reduced blanking
	.hor_total(16'd816),		// horizontal total
	.hor_addr (16'd640),		// horizontal display
	.hor_fp   (16'd56),			// horizontal front porch (+margin)
	.hor_sync (16'd32),			// horizontal sync
	.hor_bp   (16'd88),			// horizontal back porch (+margin)
	.ver_total(16'd511),		// vertical total
	.ver_addr (16'd480),		// vertical display
	.ver_fp   (16'd11),			// vertical front porch (+margin)
	.ver_sync (16'd4),			// vertical sync
	.ver_bp   (16'd16)			// vertical back porch (+margin)

`endif

`ifdef reso_800x600

		// VESA CVT : 0.48M3 40MHz 800x600@60Hz
	.hor_total(16'd1040),		// horizontal total
	.hor_addr (16'd800),		// horizontal display
	.hor_fp   (16'd40),			// horizontal front porch (+margin)
	.hor_sync (16'd80),			// horizontal sync
	.hor_bp   (16'd120),		// horizontal back porch (+margin)
	.ver_total(16'd645),		// vertical total
	.ver_addr (16'd600),		// vertical display
	.ver_fp   (16'd13),			// vertical front porch (+margin)
	.ver_sync (16'd4),			// vertical sync
	.ver_bp   (16'd28)			// vertical back porch (+margin)

`endif

`ifdef reso_1024x768

		// VESA CVT : 0.79M3 63.5MHz 1024x768@60Hz
	.hor_total(16'd1328),		// horizontal total
	.hor_addr (16'd1024),		// horizontal display
	.hor_fp   (16'd48),			// horizontal front porch (+margin)
	.hor_sync (16'd104),		// horizontal sync
	.hor_bp   (16'd152),		// horizontal back porch (+margin)
	.ver_total(16'd798),		// vertical total
	.ver_addr (16'd768),		// vertical display
	.ver_fp   (16'd3),			// vertical front porch (+margin)
	.ver_sync (16'd4),			// vertical sync
	.ver_bp   (16'd23)			// vertical back porch (+margin)

`endif


) timegen (
	.HSYNC_N(TG_HS),			// out   [CRT] #hsync
	.VSYNC_N(TG_VS),			// out   [CRT] #vsync
	.BLANK_N(TG_DE),			// out   [CRT] #blank/de

	.RST_N(RESET_N),			// in    [CRT] #reset
	.CLK(TX_CLK)				// in    [CRT] dot clock
);



dispgen dispgen(
	.D_RED(TX_RED[7:0]),		// out   [CRT] [7:0] red
	.D_GRN(TX_GRN[7:0]),		// out   [CRT] [7:0] green
	.D_BLU(TX_BLU[7:0]),		// out   [CRT] [7:0] blue
	.D_HS(TX_HS),				// out   [CRT] #hsync
	.D_VS(TX_VS),				// out   [CRT] #vsync
	.D_DE(TX_DE),				// out   [CRT] #blank/de

	.TX_HS(TG_HS),				// in    [CRT] #hsync
	.TX_VS(TG_VS),				// in    [CRT] #vsync
	.TX_DE(TG_DE),				// in    [CRT] #blank/de
	.TX_CLK(1'b1),				// in    [CRT] cke

	.RST_N(RESET_N),			// in    [CRT] #reset
	.CLK(TX_CLK)				// in    [CRT] dot clock
);



xil_dvi dvi_out (
	.DVI_TX0_N(ch2n),				// out   [TX] TX[0]-N (CML)
	.DVI_TX0_P(ch2p),				// out   [TX] TX[0]-P (CML)
	.DVI_TX1_N(ch1n),				// out   [TX] TX[1]-N (CML)
	.DVI_TX1_P(ch1p),				// out   [TX] TX[1]-P (CML)
	.DVI_TX2_N(ch0n),				// out   [TX] TX[2]-N (CML)
	.DVI_TX2_P(ch0p),				// out   [TX] TX[2]-P (CML)
	.DVI_TXC_N(ch3n),				// out   [TX] TX_CLK-N (CML)
	.DVI_TXC_P(ch3p),				// out   [TX] TX_CLK-P (CML)

	.TX_RED(TX_RED[7:0]),			// in    [TX] [7:0] red
	.TX_GRN(TX_GRN[7:0]),			// in    [TX] [7:0] green
	.TX_BLU(TX_BLU[7:0]),			// in    [TX] [7:0] blue
	.TX_HS(TX_HS),					// in    [TX] hsync
	.TX_VS(TX_VS),					// in    [TX] vsync
	.TX_C0(1'b0),					// in    [TX] c0
	.TX_C1(1'b0),					// in    [TX] c1
	.TX_C2(1'b0),					// in    [TX] c2
	.TX_C3(1'b0),					// in    [TX] c3
	.TX_DE(TX_DE),					// in    [TX] de

	.CLK(TX_CLK),					// in    [DVI] clk
	.CLKx5(!TX_CLKx5),				// in    [DVI] clk x5 : dvi ddr
	.RESET_N(RESET_N)				// in    [CRT] #reset
);




	assign ld[0] = !RST;
	assign ld[1] = !RST_N;
	assign ld[2] = !RST2_N;
	assign ld[3] = !RESET_N;
	assign ld[4] = !TX_HS;
	assign ld[5] = !TX_VS;
	assign ld[6] = !TX_DE;
	assign ld[7] = !TX_CLK;

endmodule

